import logging
import contextvars
from typing import Dict, Generator
from unittest.mock import Mock, MagicMock, patch

import pytest
from pytest import MonkeyPatch, LogCaptureFixture

from scale_gp_beta.lib.tracing.span import NoOpSpan
from scale_gp_beta.lib.tracing.trace import Trace, BaseTrace, NoOpTrace
from scale_gp_beta.lib.tracing.trace_queue_manager import TraceQueueManager


@pytest.fixture
def mock_queue_manager() -> Mock:
    manager = Mock(spec=TraceQueueManager)
    manager.report_trace_start = Mock()
    manager.report_trace_end = Mock()
    return manager


@pytest.fixture
def mock_scope_set_current_trace() -> Generator[MagicMock, None, None]:
    mock_token = Mock(spec=contextvars.Token)
    with patch("scale_gp_beta.lib.tracing.trace.Scope.set_current_trace", return_value=mock_token) as mock_set:
        yield mock_set

@pytest.fixture
def mock_scope_set_current_span() -> Generator[MagicMock, None, None]:
    mock_token = Mock(spec=contextvars.Token)
    with patch("scale_gp_beta.lib.tracing.span.Scope.set_current_span", return_value=mock_token) as mock_set:
        yield mock_set


@pytest.fixture
def mock_scope_reset_current_trace() -> Generator[MagicMock, None, None]:
    with patch("scale_gp_beta.lib.tracing.trace.Scope.reset_current_trace") as mock_reset:
        yield mock_reset


@pytest.fixture
def mock_scope_reset_current_span() -> Generator[MagicMock, None, None]:
    with patch("scale_gp_beta.lib.tracing.span.Scope.reset_current_span") as mock_reset:
        yield mock_reset


@pytest.fixture
def patched_trace_utils(monkeypatch: MonkeyPatch) -> Dict[str, Mock]:
    """Mocks utility functions for predictable values related to traces."""
    mock_gen_trace_id = Mock(return_value="test-trace-id")
    monkeypatch.setattr("scale_gp_beta.lib.tracing.trace.generate_trace_id", mock_gen_trace_id)
    return {"generate_trace_id": mock_gen_trace_id}


class TestBaseTrace:
    def test_initialization_defaults(self, mock_queue_manager: Mock) -> None:
        trace = BaseTrace(queue_manager=mock_queue_manager, trace_id="test-trace-id", root_span=NoOpSpan("test-root"))
        assert trace.trace_id == "test-trace-id"
        assert trace.queue_manager == mock_queue_manager
        assert trace._in_progress is False
        assert trace._contextvar_token is None

    def test_initialization_with_custom_trace_id(self, mock_queue_manager: Mock) -> None:
        custom_id = "my-custom-trace-id"
        trace = BaseTrace(queue_manager=mock_queue_manager, trace_id=custom_id, root_span=NoOpSpan("test-root"))
        assert trace.trace_id == custom_id
        assert trace.queue_manager == mock_queue_manager

    def test_context_manager_calls_start_end(self, mock_queue_manager: Mock) -> None:
        trace = BaseTrace(queue_manager=mock_queue_manager, trace_id="test-trace", root_span=NoOpSpan("test-root"))
        trace.start = Mock()
        trace.end = Mock()

        with trace as t:
            assert t == trace
            trace.start.assert_called_once()
            trace.end.assert_not_called()
        trace.end.assert_called_once()

    def test_context_manager_propagates_exception(self, mock_queue_manager: Mock) -> None:
        trace = BaseTrace(queue_manager=mock_queue_manager, trace_id="test-trace", root_span=NoOpSpan("test-root"))
        trace.end = Mock()

        with pytest.raises(ValueError, match="Test error in trace"):
            with trace:
                raise ValueError("Test error in trace")
        trace.end.assert_called_once()


class TestNoOpTrace:
    def test_initialization(self, patched_trace_utils: Dict[str, Mock]) -> None:
        trace = NoOpTrace(trace_id="noop-trace", name="noop-trace-name")
        assert trace.trace_id == "noop-trace"
        assert trace.queue_manager is None
        assert trace._in_progress is False
        assert trace._contextvar_token is None

        assert trace.root_span.name == "noop-trace-name"
        assert trace.root_span._contextvar_token is None
        patched_trace_utils["generate_trace_id"].assert_not_called()


    def test_initialization_default_id(self, patched_trace_utils: Dict[str, Mock]) -> None:
        trace = NoOpTrace(name="noop-trace-name")
        assert trace.trace_id == "test-trace-id"
        patched_trace_utils["generate_trace_id"].assert_called_once()


class TestTrace:
    def test_initialization(self, mock_queue_manager: Mock, patched_trace_utils: Dict[str, Mock]) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="op-trace-id", name="op-trace-name", span_id="root-span-id")
        assert trace.trace_id == "op-trace-id"
        assert trace.queue_manager == mock_queue_manager
        assert trace.root_span.name == "op-trace-name"
        assert trace.root_span.span_id == "root-span-id"
        patched_trace_utils["generate_trace_id"].assert_not_called()

    def test_start(
        self,
        mock_queue_manager: Mock,
        mock_scope_set_current_trace: MagicMock,
        mock_scope_set_current_span: MagicMock,
        mock_scope_reset_current_trace: MagicMock, # noqa: ARG002
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="start-test-trace", name="start-test-trace", span_id="root-span-id")
        trace.start()

        assert trace._in_progress is True
        mock_queue_manager.report_trace_start.assert_called_once_with(trace)
        mock_scope_set_current_trace.assert_called_once_with(trace)
        assert trace._contextvar_token == mock_scope_set_current_trace.return_value
        mock_scope_set_current_span.assert_called_once_with(trace.root_span)
        assert trace.root_span._contextvar_token == mock_scope_set_current_span.return_value

    def test_start_idempotency(
        self,
        mock_queue_manager: Mock,
        mock_scope_set_current_trace: MagicMock,
        mock_scope_set_current_span: MagicMock,
        caplog: LogCaptureFixture,
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="idem-start-trace", name="idem-start-trace", span_id="root-span-id")
        trace.start()

        mock_queue_manager.reset_mock()
        mock_scope_set_current_trace.reset_mock()
        mock_scope_set_current_span.reset_mock()

        with caplog.at_level(logging.WARNING):
            trace.start()

        assert f"Trace already started: {trace.trace_id}" in caplog.text
        mock_queue_manager.report_trace_start.assert_not_called()
        mock_scope_set_current_trace.assert_not_called()
        mock_scope_set_current_span.assert_not_called()

    def test_end_normal_flow(
        self,
        mock_queue_manager: Mock,
        mock_scope_set_current_trace: MagicMock,
        mock_scope_set_current_span: MagicMock,
        mock_scope_reset_current_trace: MagicMock,
        mock_scope_reset_current_span: MagicMock,
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="end-test-trace", name="end-test-trace", span_id="root-span-id")
        trace.start()

        expected_token = mock_scope_set_current_trace.return_value
        expected_span_token = mock_scope_set_current_span.return_value
        assert trace._contextvar_token == expected_token
        assert trace._in_progress is True
        assert trace.root_span._contextvar_token == expected_span_token

        trace.end()

        assert trace._in_progress is False
        mock_queue_manager.report_trace_end.assert_called_once_with(trace)
        mock_queue_manager.report_span_end.assert_called_once_with(trace.root_span)
        mock_scope_reset_current_trace.assert_called_once_with(expected_token)
        mock_scope_reset_current_span.assert_called_once_with(expected_span_token)
        assert trace._contextvar_token is None
        assert trace.root_span._contextvar_token is None

    def test_end_when_not_in_progress(
        self,
        mock_queue_manager: Mock,
        mock_scope_reset_current_trace: MagicMock,
        mock_scope_reset_current_span: MagicMock,
        caplog: LogCaptureFixture,
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="not-active-trace", name="not-active-trace", span_id="root-span-id")

        with caplog.at_level(logging.WARNING):
            trace.end()

        assert f"Ending trace which is not active: {trace.trace_id}" in caplog.text
        mock_queue_manager.report_trace_end.assert_not_called()
        mock_queue_manager.report_span_end.assert_not_called()
        mock_scope_reset_current_trace.assert_not_called()
        mock_scope_reset_current_span.assert_not_called()

    def test_end_when_token_is_none(
        self,
        mock_queue_manager: Mock,
        mock_scope_reset_current_trace: MagicMock,
        mock_scope_reset_current_span: MagicMock,
        caplog: LogCaptureFixture,
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="no-token-trace", name="no-token-trace", span_id="root-span-id")
        trace._in_progress = True
        trace._contextvar_token = None

        with caplog.at_level(logging.WARNING):
            trace.end()

        assert f"Ending trace which is not active: {trace.trace_id}, contextvar_token not set" in caplog.text
        mock_queue_manager.report_trace_end.assert_not_called()
        mock_queue_manager.report_span_end.assert_not_called()
        mock_scope_reset_current_trace.assert_not_called()
        mock_scope_reset_current_span.assert_not_called()

    def test_context_manager_operational_trace(
        self,
        mock_queue_manager: Mock,
        mock_scope_set_current_trace: MagicMock,
        mock_scope_set_current_span: MagicMock,
        mock_scope_reset_current_trace: MagicMock,
        mock_scope_reset_current_span: MagicMock,
        patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, trace_id="ctx-op-trace", name="ctx-op-trace", span_id="root-span-id")
        expected_token = mock_scope_set_current_trace.return_value
        expected_span_token = mock_scope_set_current_span.return_value

        with trace as t:
            assert t == trace
            assert trace._in_progress is True
            mock_queue_manager.report_trace_start.assert_called_once_with(trace)
            mock_queue_manager.report_span_start.assert_called_once_with(trace.root_span)
            mock_scope_set_current_trace.assert_called_once_with(trace)
            mock_scope_set_current_span.assert_called_once_with(trace.root_span)
            assert trace._contextvar_token == expected_token
            assert trace.root_span._contextvar_token == expected_span_token

        assert trace._in_progress is False
        assert trace.root_span._contextvar_token is None
        mock_queue_manager.report_trace_end.assert_called_once_with(trace)
        mock_queue_manager.report_span_end.assert_called_once_with(trace.root_span)
        mock_scope_reset_current_trace.assert_called_once_with(expected_token)
        mock_scope_reset_current_span.assert_called_once_with(expected_span_token)
        assert trace._contextvar_token is None

    def test_updating_trace_status(
            self,
            mock_queue_manager: Mock,
            patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, name="request_123")
        with pytest.raises(AttributeError):
            trace.status = "ERROR"  # type: ignore[attr-defined]
        assert trace.root_span.status == "SUCCESS"


    def test_setting_trace_metadata(
            self,
            mock_queue_manager: Mock,
            patched_trace_utils: Dict[str, Mock] # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, name="request_123")
        trace.metadata = {"foo": "bar"}
        trace.metadata["other"] = "baz"
        assert trace.root_span.metadata == {"foo": "bar", "other": "baz"}

    def test_setting_trace_input(
            self,
            mock_queue_manager: Mock,
            patched_trace_utils: Dict[str, Mock]  # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, name="request_123")
        trace.input = {"foo": "bar"}
        assert trace.root_span.input == {"foo": "bar"}

    def test_setting_trace_output(
            self,
            mock_queue_manager: Mock,
            patched_trace_utils: Dict[str, Mock]  # noqa: ARG002
    ) -> None:
        trace = Trace(queue_manager=mock_queue_manager, name="request_123")
        trace.output = {"foo": "bar"}
        assert trace.root_span.output == {"foo": "bar"}

    def test_set_error(self, mock_queue_manager: Mock) -> None:
        trace = Trace(queue_manager=mock_queue_manager, name="request_123")
        e = AttributeError("this is a test")
        assert trace.metadata == {}
        trace.set_error(exception=e)

        assert trace.metadata == {"error": True, "error_message": "this is a test", "error_type": "AttributeError"}



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["EvaluationListParams"]


class EvaluationListParams(TypedDict, total=False):
    ending_before: str

    include_archived: bool

    limit: int

    name: str

    sort_by: str

    sort_order: Literal["asc", "desc"]

    starting_after: str

    tags: SequenceNotStr[str]

    views: List[Literal["tasks"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .item_locator import ItemLocator

__all__ = ["ComponentParam"]


class ComponentParam(TypedDict, total=False):
    data: Required[ItemLocator]
    """
    A pointer to the data in each evaluation item to be displayed within the
    component
    """

    label: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["DatasetItemUpdateParams"]


class DatasetItemUpdateParams(TypedDict, total=False):
    data: Required[Dict[str, object]]
    """Updated dataset item data"""

    files: Dict[str, str]
    """Files to be associated to the dataset"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

from .approval_status import ApprovalStatus
from .assessment_type import AssessmentType

__all__ = ["SpanAssessmentCreateParams"]


class SpanAssessmentCreateParams(TypedDict, total=False):
    assessment_type: Required[AssessmentType]
    """Type of assessment"""

    span_id: Required[str]
    """The ID of the span this assessment is attached to"""

    trace_id: Required[str]
    """The ID of the trace this assessment is attached to"""

    approval: ApprovalStatus
    """Approval status (approved/rejected)"""

    comment: str
    """Raw text feedback"""

    metadata: Dict[str, object]
    """Arbitrary JSON object for additional data"""

    overwrite: Dict[str, object]
    """User corrections to span output"""

    rating: int
    """Numerical rating (1-5)"""

    rubric: Dict[str, str]
    """Rule key-value pairs for rubric evaluation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, TypedDict

__all__ = ["VectorStoreUpsertParams", "Vector"]


class VectorStoreUpsertParams(TypedDict, total=False):
    vectors: Required[Iterable[Vector]]
    """Array of documents to upsert"""


class Vector(TypedDict, total=False):
    """A document for insert/upsert operations."""

    id: str
    """Unique document ID (auto-generated if not provided)"""

    metadata: Dict[str, object]
    """Key-value metadata"""

    text: str
    """Text content to be embedded"""

    vector: Iterable[float]
    """Pre-computed embedding vector"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["VectorStoreQueryParams"]


class VectorStoreQueryParams(TypedDict, total=False):
    text: Required[str]
    """Text query for automatic embedding (required)"""

    filter: Dict[str, object]
    """Metadata filter expression"""

    include_vectors: bool
    """Include embedding vectors in response"""

    query_type: Literal["semantic", "lexical", "hybrid"]
    """Query type: semantic, lexical, or hybrid"""

    rerank: bool
    """Enable reranking of search results"""

    rerank_model: str
    """Reranking model to use (uses system default if not specified)"""

    rerank_top_n: int
    """Number of results after reranking (defaults to top_k)"""

    top_k: int
    """Number of search results to return"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import TYPE_CHECKING, Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .approval_status import ApprovalStatus
from .assessment_type import AssessmentType

__all__ = ["SpanAssessment"]


class SpanAssessment(BaseModel):
    """Response model for span assessment"""

    assessment_id: str
    """Unique identifier for the assessment"""

    assessment_type: AssessmentType
    """Type of assessment"""

    created_by: str
    """User who submitted the assessment"""

    span_id: str
    """The span this assessment is attached to"""

    trace_id: str
    """The trace this assessment is attached to"""

    account_id: Optional[str] = None
    """Account this assessment belongs to"""

    approval: Optional[ApprovalStatus] = None
    """Approval status (approved/rejected)"""

    comment: Optional[str] = None
    """Raw text feedback"""

    created_at: Optional[datetime] = None
    """When this assessment was created"""

    metadata: Optional[Dict[str, object]] = None
    """Arbitrary JSON object for additional data"""

    object: Optional[Literal["span.assessment"]] = None

    overwrite: Optional[Dict[str, builtins.object]] = None
    """User corrections to span output"""

    rating: Optional[int] = None
    """Numerical rating (1-5)"""

    rubric: Optional[Dict[str, str]] = None
    """Rule key-value pairs for rubric evaluation"""

    updated_at: Optional[datetime] = None
    """When this assessment was last updated"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, builtins.object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> builtins.object: ...
    else:
        __pydantic_extra__: Dict[str, builtins.object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["SpanType"]

SpanType: TypeAlias = Literal[
    "TEXT_INPUT",
    "TEXT_OUTPUT",
    "COMPLETION_INPUT",
    "COMPLETION",
    "KB_RETRIEVAL",
    "KB_INPUT",
    "RERANKING",
    "EXTERNAL_ENDPOINT",
    "PROMPT_ENGINEERING",
    "DOCUMENT_INPUT",
    "MAP_REDUCE",
    "DOCUMENT_SEARCH",
    "DOCUMENT_PROMPT",
    "CUSTOM",
    "CODE_EXECUTION",
    "DATA_MANIPULATION",
    "EVALUATION",
    "FILE_RETRIEVAL",
    "KB_ADD_CHUNK",
    "KB_MANAGEMENT",
    "GUARDRAIL",
    "TRACER",
    "AGENT_TRACER",
    "AGENT_WORKFLOW",
    "STANDALONE",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["EvaluationItemExportResponse"]


class EvaluationItemExportResponse(BaseModel):
    """
    Response model for exporting evaluation items.
    This class represents the response when users export evaluation items.
    It contains either a signed URL to download the exported data from object storage,
    or the actual content bytes when direct download is used (in environments where object storage is not configured).
    """

    filename: str
    """The name of the exported file"""

    content: Optional[object] = None
    """The raw file content as bytes, used when direct download is enabled"""

    signed_url: Optional[str] = None
    """Pre-signed URL to download the file from object storage, if applicable"""



import time
import logging
from queue import Empty, Queue
from typing import TYPE_CHECKING, List
from typing_extensions import TypeAlias

from scale_gp_beta import SGPClient
from scale_gp_beta._exceptions import APIError
from scale_gp_beta.types.span_upsert_batch_params import Item as SpanCreateRequest

from .exceptions import ParamsCreationError

if TYPE_CHECKING:
    from .tracing import Span

INITIAL_BACKOFF = 0.4
MAX_BACKOFF = 20

log: logging.Logger = logging.getLogger(__name__)

SpanRequestBatch: TypeAlias = List[SpanCreateRequest]


class TraceExporter:
    def __init__(
        self,
        max_batch_size: int,
        max_retries: int,
        backoff: float = INITIAL_BACKOFF,
        max_backoff: float = MAX_BACKOFF,
    ):
        self.max_batch_size = max_batch_size
        self.max_retries = max_retries
        self.backoff = backoff
        self.max_backoff = max_backoff

    def export_span(self, client: SGPClient, span: "Span") -> None:
        # only upsert endpoint is batch upsert, so we work in batches
        batch: SpanRequestBatch = self._create_one_span_batch(span)

        log.info(f"Exporting single span with id {span.span_id}")
        self._export_batch(batch, client)

    def export(self, client: SGPClient, queue: "Queue[Span]") -> None:
        # this is also thread safe, two threads can call this method with the same queue at once
        # the ordering of the requests might be randomly split between the two threads, but they should all be picked up
        batches: list[SpanRequestBatch] = self._create_batches(queue)

        if not batches:
            log.debug("No new span batches to export")
            return

        log.info(f"Exporting {len(batches)} span batches")

        for batch in batches:
            self._export_batch(batch, client)

    def _export_batch(self, batch: SpanRequestBatch, client: SGPClient) -> None:
        attempts_remaining = self.max_retries
        backoff_delay = self.backoff
        while attempts_remaining > 0:
            attempts_remaining -= 1
            try:
                client.spans.upsert_batch(items=batch)
                return
            except APIError as e:
                log.warning(
                    f"API error occurred while exporting batch: {e.message}, attempts remaining: {attempts_remaining}"
                )
                if attempts_remaining == 0:
                    continue
                time.sleep(backoff_delay)
                backoff_delay = min(backoff_delay * 2, self.max_backoff)

        log.error(f"Failed to export span batch after {self.max_retries} attempts, dropping...")

    def _create_batches(self, queue: "Queue[Span]") -> "list[SpanRequestBatch]":
        """Drain the queue and return a list of batches"""
        batches: list[SpanRequestBatch] = []

        while True:
            span_batch: SpanRequestBatch = []

            while len(span_batch) < self.max_batch_size and queue.qsize() > 0:
                try:
                    span: "Span" = queue.get_nowait()
                    self._add_span_to_batch(span, span_batch)
                except Empty:
                    break

            if not span_batch:
                break
            batches.append(span_batch)

        return batches

    def _create_one_span_batch(self, span: "Span") -> SpanRequestBatch:
        span_batch: SpanRequestBatch = []
        self._add_span_to_batch(span, span_batch)

        return span_batch

    def _add_span_to_batch(self, span: "Span", span_batch: SpanRequestBatch) -> None:
        try:
            span_request = span.to_request_params()
            span_batch.append(span_request)
        except ParamsCreationError as e:
            log.warning(f"ParamsCreationError: dropping span: {span}\n{e}")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from scale_gp_beta import SGPClient, AsyncSGPClient
from scale_gp_beta.types import (
    File,
    FileDeleteResponse,
    FileImportFromCloudResponse,
)
from scale_gp_beta.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        file = client.files.create(
            file=b"raw file contents",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.create(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.create(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        file = client.files.retrieve(
            "file_id",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.retrieve(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.retrieve(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.files.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        file = client.files.update(
            file_id="file_id",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        file = client.files.update(
            file_id="file_id",
            tags={"foo": "bar"},
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.update(
            file_id="file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.update(
            file_id="file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.files.with_raw_response.update(
                file_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        file = client.files.list()
        assert_matches_type(SyncCursorPage[File], file, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        file = client.files.list(
            ending_before="ending_before",
            filename="filename",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(SyncCursorPage[File], file, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(SyncCursorPage[File], file, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(SyncCursorPage[File], file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        file = client.files.delete(
            "file_id",
        )
        assert_matches_type(FileDeleteResponse, file, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.delete(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(FileDeleteResponse, file, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.delete(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(FileDeleteResponse, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            client.files.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_import_from_cloud(self, client: SGPClient) -> None:
        file = client.files.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        )
        assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

    @parametrize
    def test_raw_response_import_from_cloud(self, client: SGPClient) -> None:
        response = client.files.with_raw_response.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = response.parse()
        assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

    @parametrize
    def test_streaming_response_import_from_cloud(self, client: SGPClient) -> None:
        with client.files.with_streaming_response.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = response.parse()
            assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncFiles:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.create(
            file=b"raw file contents",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.create(
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.create(
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.retrieve(
            "file_id",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.retrieve(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.retrieve(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.files.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.update(
            file_id="file_id",
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.update(
            file_id="file_id",
            tags={"foo": "bar"},
        )
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.update(
            file_id="file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(File, file, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.update(
            file_id="file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(File, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.files.with_raw_response.update(
                file_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.list()
        assert_matches_type(AsyncCursorPage[File], file, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.list(
            ending_before="ending_before",
            filename="filename",
            limit=1,
            sort_by="sort_by",
            sort_order="asc",
            starting_after="starting_after",
        )
        assert_matches_type(AsyncCursorPage[File], file, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(AsyncCursorPage[File], file, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(AsyncCursorPage[File], file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.delete(
            "file_id",
        )
        assert_matches_type(FileDeleteResponse, file, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.delete(
            "file_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(FileDeleteResponse, file, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.delete(
            "file_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(FileDeleteResponse, file, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `file_id` but received ''"):
            await async_client.files.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_import_from_cloud(self, async_client: AsyncSGPClient) -> None:
        file = await async_client.files.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        )
        assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

    @parametrize
    async def test_raw_response_import_from_cloud(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.files.with_raw_response.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        file = await response.parse()
        assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

    @parametrize
    async def test_streaming_response_import_from_cloud(self, async_client: AsyncSGPClient) -> None:
        async with async_client.files.with_streaming_response.import_from_cloud(
            files=[
                {
                    "container": "container",
                    "file_type": "file_type",
                    "filename": "filename",
                    "filepath": "filepath",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            file = await response.parse()
            assert_matches_type(FileImportFromCloudResponse, file, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import datetime
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo
from .span_status import SpanStatus

__all__ = ["SpanUpdateParams"]


class SpanUpdateParams(TypedDict, total=False):
    end_timestamp: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]

    metadata: Dict[str, object]

    name: str

    output: Dict[str, object]

    status: SpanStatus



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .span_type import SpanType
from .span_status import SpanStatus
from .shared.identity import Identity

__all__ = ["Span"]


class Span(BaseModel):
    id: str

    account_id: str

    name: str

    start_timestamp: datetime

    trace_id: str
    """id for grouping traces together, uuid is recommended"""

    application_interaction_id: Optional[str] = None
    """The interaction ID this span belongs to"""

    application_variant_id: Optional[str] = None
    """The id of the application variant this span belongs to"""

    created_by: Optional[Identity] = None
    """The identity that created the entity."""

    end_timestamp: Optional[datetime] = None

    group_id: Optional[str] = None
    """Reference to a group_id"""

    input: Optional[Dict[str, object]] = None

    metadata: Optional[Dict[str, object]] = None

    object: Optional[Literal["span"]] = None

    output: Optional[Dict[str, builtins.object]] = None

    parent_id: Optional[str] = None
    """Reference to a parent span_id"""

    status: Optional[SpanStatus] = None

    type: Optional[SpanType] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["VectorStoreVectorsResponse", "Vector"]


class Vector(BaseModel):
    """A document in list/query responses."""

    id: str
    """Document ID"""

    metadata: Optional[Dict[str, object]] = None
    """Key-value metadata"""

    score: Optional[float] = None
    """Similarity score (for query results)"""

    text: Optional[str] = None
    """Text content"""

    vector: Optional[List[float]] = None
    """Embedding vector (if requested)"""


class VectorStoreVectorsResponse(BaseModel):
    """Response for list operation."""

    vectors: List[Vector]
    """Array of documents"""

    next_cursor: Optional[str] = None
    """Pagination cursor for next page"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ModelListParams"]


class ModelListParams(TypedDict, total=False):
    ending_before: str

    limit: int

    model_vendor: Literal[
        "openai",
        "cohere",
        "vertex_ai",
        "anthropic",
        "azure",
        "gemini",
        "launch",
        "llmengine",
        "model_zoo",
        "bedrock",
        "xai",
        "fireworks_ai",
    ]

    name: str

    sort_by: str

    sort_order: Literal["asc", "desc"]

    starting_after: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ModelDefinition"]


class ModelDefinition(BaseModel):
    name: str = FieldInfo(alias="model_name")
    """model name, for example `gpt-4o`"""

    type: Literal["generic", "completion", "chat_completion"] = FieldInfo(alias="model_type")
    """model type, for example `chat_completion`"""

    vendor: Literal[
        "openai",
        "cohere",
        "vertex_ai",
        "anthropic",
        "azure",
        "gemini",
        "launch",
        "llmengine",
        "model_zoo",
        "bedrock",
        "xai",
        "fireworks_ai",
    ] = FieldInfo(alias="model_vendor")
    """model vendor, for example `openai`"""

    availability: Optional[Literal["unknown", "available", "unavailable"]] = FieldInfo(
        alias="model_availability", default=None
    )
    """model availability indicating availability status, for example `available`"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .model_definition import ModelDefinition

__all__ = ["CompletionModelsResponse"]


class CompletionModelsResponse(BaseModel):
    items: List[ModelDefinition]

    object: Optional[Literal["list"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["VectorStoreConfigureParams"]


class VectorStoreConfigureParams(TypedDict, total=False):
    indexed_metadata_fields: Required[Dict[str, Literal["string", "number", "boolean", "object", "list"]]]
    """Dictionary mapping metadata field names to their types.

    Only STRING, NUMBER, and BOOLEAN types can be indexed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..types import (
    SpanType,
    SpanStatus,
    span_batch_params,
    span_create_params,
    span_search_params,
    span_update_params,
    span_upsert_batch_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from ..types.span import Span
from .._base_client import AsyncPaginator, make_request_options
from ..types.span_type import SpanType
from ..types.span_status import SpanStatus
from ..types.span_batch_response import SpanBatchResponse
from ..types.span_upsert_batch_response import SpanUpsertBatchResponse

__all__ = ["SpansResource", "AsyncSpansResource"]


class SpansResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SpansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return SpansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return SpansResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        start_timestamp: Union[str, datetime],
        trace_id: str,
        id: str | Omit = omit,
        application_interaction_id: str | Omit = omit,
        application_variant_id: str | Omit = omit,
        end_timestamp: Union[str, datetime] | Omit = omit,
        group_id: str | Omit = omit,
        input: Dict[str, object] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        output: Dict[str, object] | Omit = omit,
        parent_id: str | Omit = omit,
        status: SpanStatus | Omit = omit,
        type: SpanType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Create Span

        Args:
          trace_id: id for grouping traces together, uuid is recommended

          id: The id of the span

          application_interaction_id: The optional application interaction ID this span belongs to

          application_variant_id: The optional application variant ID this span belongs to

          group_id: Reference to a group_id

          parent_id: Reference to a parent span_id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v5/spans",
            body=maybe_transform(
                {
                    "name": name,
                    "start_timestamp": start_timestamp,
                    "trace_id": trace_id,
                    "id": id,
                    "application_interaction_id": application_interaction_id,
                    "application_variant_id": application_variant_id,
                    "end_timestamp": end_timestamp,
                    "group_id": group_id,
                    "input": input,
                    "metadata": metadata,
                    "output": output,
                    "parent_id": parent_id,
                    "status": status,
                    "type": type,
                },
                span_create_params.SpanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    def retrieve(
        self,
        span_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Get Span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_id:
            raise ValueError(f"Expected a non-empty value for `span_id` but received {span_id!r}")
        return self._get(
            f"/v5/spans/{span_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    def update(
        self,
        span_id: str,
        *,
        end_timestamp: Union[str, datetime] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        name: str | Omit = omit,
        output: Dict[str, object] | Omit = omit,
        status: SpanStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Update Span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_id:
            raise ValueError(f"Expected a non-empty value for `span_id` but received {span_id!r}")
        return self._patch(
            f"/v5/spans/{span_id}",
            body=maybe_transform(
                {
                    "end_timestamp": end_timestamp,
                    "metadata": metadata,
                    "name": name,
                    "output": output,
                    "status": status,
                },
                span_update_params.SpanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    def batch(
        self,
        *,
        items: Iterable[span_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanBatchResponse:
        """
        Create Spans in Batch

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v5/spans/batch",
            body=maybe_transform({"items": items}, span_batch_params.SpanBatchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanBatchResponse,
        )

    def search(
        self,
        *,
        ending_before: str | Omit = omit,
        from_ts: Union[str, datetime] | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        to_ts: Union[str, datetime] | Omit = omit,
        acp_types: SequenceNotStr[str] | Omit = omit,
        agentex_agent_ids: SequenceNotStr[str] | Omit = omit,
        agentex_agent_names: SequenceNotStr[str] | Omit = omit,
        application_variant_ids: SequenceNotStr[str] | Omit = omit,
        assessment_types: SequenceNotStr[str] | Omit = omit,
        excluded_span_ids: SequenceNotStr[str] | Omit = omit,
        excluded_trace_ids: SequenceNotStr[str] | Omit = omit,
        extra_metadata: Dict[str, object] | Omit = omit,
        group_id: str | Omit = omit,
        max_duration_ms: int | Omit = omit,
        min_duration_ms: int | Omit = omit,
        names: SequenceNotStr[str] | Omit = omit,
        parents_only: bool | Omit = omit,
        search_texts: SequenceNotStr[str] | Omit = omit,
        span_ids: SequenceNotStr[str] | Omit = omit,
        statuses: List[SpanStatus] | Omit = omit,
        trace_ids: SequenceNotStr[str] | Omit = omit,
        types: List[SpanType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[Span]:
        """
        Search and list spans

        Args:
          from_ts: The starting (oldest) timestamp in ISO format.

          to_ts: The ending (most recent) timestamp in ISO format.

          acp_types: Filter by ACP types

          agentex_agent_ids: Filter by Agentex agent IDs

          agentex_agent_names: Filter by Agentex agent names

          application_variant_ids: Filter by application variant IDs

          assessment_types: Filter spans by traces that have assessments of these types

          excluded_span_ids: List of span IDs to exclude from results

          excluded_trace_ids: List of trace IDs to exclude from results

          extra_metadata: Filter on custom metadata key-value pairs

          group_id: Filter by group ID

          max_duration_ms: Maximum span duration in milliseconds (inclusive)

          min_duration_ms: Minimum span duration in milliseconds (inclusive)

          names: Filter by trace/span name

          parents_only: Only fetch spans that are the top-level (ie. have no parent_id)

          search_texts: Free text search across span input and output fields

          span_ids: Filter by span IDs

          statuses: Filter on span status

          trace_ids: Filter by trace IDs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/spans/search",
            page=SyncCursorPage[Span],
            body=maybe_transform(
                {
                    "acp_types": acp_types,
                    "agentex_agent_ids": agentex_agent_ids,
                    "agentex_agent_names": agentex_agent_names,
                    "application_variant_ids": application_variant_ids,
                    "assessment_types": assessment_types,
                    "excluded_span_ids": excluded_span_ids,
                    "excluded_trace_ids": excluded_trace_ids,
                    "extra_metadata": extra_metadata,
                    "group_id": group_id,
                    "max_duration_ms": max_duration_ms,
                    "min_duration_ms": min_duration_ms,
                    "names": names,
                    "parents_only": parents_only,
                    "search_texts": search_texts,
                    "span_ids": span_ids,
                    "statuses": statuses,
                    "trace_ids": trace_ids,
                    "types": types,
                },
                span_search_params.SpanSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "from_ts": from_ts,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "to_ts": to_ts,
                    },
                    span_search_params.SpanSearchParams,
                ),
            ),
            model=Span,
            method="post",
        )

    def upsert_batch(
        self,
        *,
        items: Iterable[span_upsert_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanUpsertBatchResponse:
        """
        Upsert Spans in Batch

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/v5/spans/batch",
            body=maybe_transform({"items": items}, span_upsert_batch_params.SpanUpsertBatchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanUpsertBatchResponse,
        )


class AsyncSpansResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSpansResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#accessing-raw-response-data-eg-headers
        """
        return AsyncSpansResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpansResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python-beta#with_streaming_response
        """
        return AsyncSpansResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        start_timestamp: Union[str, datetime],
        trace_id: str,
        id: str | Omit = omit,
        application_interaction_id: str | Omit = omit,
        application_variant_id: str | Omit = omit,
        end_timestamp: Union[str, datetime] | Omit = omit,
        group_id: str | Omit = omit,
        input: Dict[str, object] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        output: Dict[str, object] | Omit = omit,
        parent_id: str | Omit = omit,
        status: SpanStatus | Omit = omit,
        type: SpanType | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Create Span

        Args:
          trace_id: id for grouping traces together, uuid is recommended

          id: The id of the span

          application_interaction_id: The optional application interaction ID this span belongs to

          application_variant_id: The optional application variant ID this span belongs to

          group_id: Reference to a group_id

          parent_id: Reference to a parent span_id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v5/spans",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "start_timestamp": start_timestamp,
                    "trace_id": trace_id,
                    "id": id,
                    "application_interaction_id": application_interaction_id,
                    "application_variant_id": application_variant_id,
                    "end_timestamp": end_timestamp,
                    "group_id": group_id,
                    "input": input,
                    "metadata": metadata,
                    "output": output,
                    "parent_id": parent_id,
                    "status": status,
                    "type": type,
                },
                span_create_params.SpanCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    async def retrieve(
        self,
        span_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Get Span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_id:
            raise ValueError(f"Expected a non-empty value for `span_id` but received {span_id!r}")
        return await self._get(
            f"/v5/spans/{span_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    async def update(
        self,
        span_id: str,
        *,
        end_timestamp: Union[str, datetime] | Omit = omit,
        metadata: Dict[str, object] | Omit = omit,
        name: str | Omit = omit,
        output: Dict[str, object] | Omit = omit,
        status: SpanStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Span:
        """
        Update Span

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not span_id:
            raise ValueError(f"Expected a non-empty value for `span_id` but received {span_id!r}")
        return await self._patch(
            f"/v5/spans/{span_id}",
            body=await async_maybe_transform(
                {
                    "end_timestamp": end_timestamp,
                    "metadata": metadata,
                    "name": name,
                    "output": output,
                    "status": status,
                },
                span_update_params.SpanUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Span,
        )

    async def batch(
        self,
        *,
        items: Iterable[span_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanBatchResponse:
        """
        Create Spans in Batch

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v5/spans/batch",
            body=await async_maybe_transform({"items": items}, span_batch_params.SpanBatchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanBatchResponse,
        )

    def search(
        self,
        *,
        ending_before: str | Omit = omit,
        from_ts: Union[str, datetime] | Omit = omit,
        limit: int | Omit = omit,
        sort_by: str | Omit = omit,
        sort_order: Literal["asc", "desc"] | Omit = omit,
        starting_after: str | Omit = omit,
        to_ts: Union[str, datetime] | Omit = omit,
        acp_types: SequenceNotStr[str] | Omit = omit,
        agentex_agent_ids: SequenceNotStr[str] | Omit = omit,
        agentex_agent_names: SequenceNotStr[str] | Omit = omit,
        application_variant_ids: SequenceNotStr[str] | Omit = omit,
        assessment_types: SequenceNotStr[str] | Omit = omit,
        excluded_span_ids: SequenceNotStr[str] | Omit = omit,
        excluded_trace_ids: SequenceNotStr[str] | Omit = omit,
        extra_metadata: Dict[str, object] | Omit = omit,
        group_id: str | Omit = omit,
        max_duration_ms: int | Omit = omit,
        min_duration_ms: int | Omit = omit,
        names: SequenceNotStr[str] | Omit = omit,
        parents_only: bool | Omit = omit,
        search_texts: SequenceNotStr[str] | Omit = omit,
        span_ids: SequenceNotStr[str] | Omit = omit,
        statuses: List[SpanStatus] | Omit = omit,
        trace_ids: SequenceNotStr[str] | Omit = omit,
        types: List[SpanType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Span, AsyncCursorPage[Span]]:
        """
        Search and list spans

        Args:
          from_ts: The starting (oldest) timestamp in ISO format.

          to_ts: The ending (most recent) timestamp in ISO format.

          acp_types: Filter by ACP types

          agentex_agent_ids: Filter by Agentex agent IDs

          agentex_agent_names: Filter by Agentex agent names

          application_variant_ids: Filter by application variant IDs

          assessment_types: Filter spans by traces that have assessments of these types

          excluded_span_ids: List of span IDs to exclude from results

          excluded_trace_ids: List of trace IDs to exclude from results

          extra_metadata: Filter on custom metadata key-value pairs

          group_id: Filter by group ID

          max_duration_ms: Maximum span duration in milliseconds (inclusive)

          min_duration_ms: Minimum span duration in milliseconds (inclusive)

          names: Filter by trace/span name

          parents_only: Only fetch spans that are the top-level (ie. have no parent_id)

          search_texts: Free text search across span input and output fields

          span_ids: Filter by span IDs

          statuses: Filter on span status

          trace_ids: Filter by trace IDs

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v5/spans/search",
            page=AsyncCursorPage[Span],
            body=maybe_transform(
                {
                    "acp_types": acp_types,
                    "agentex_agent_ids": agentex_agent_ids,
                    "agentex_agent_names": agentex_agent_names,
                    "application_variant_ids": application_variant_ids,
                    "assessment_types": assessment_types,
                    "excluded_span_ids": excluded_span_ids,
                    "excluded_trace_ids": excluded_trace_ids,
                    "extra_metadata": extra_metadata,
                    "group_id": group_id,
                    "max_duration_ms": max_duration_ms,
                    "min_duration_ms": min_duration_ms,
                    "names": names,
                    "parents_only": parents_only,
                    "search_texts": search_texts,
                    "span_ids": span_ids,
                    "statuses": statuses,
                    "trace_ids": trace_ids,
                    "types": types,
                },
                span_search_params.SpanSearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ending_before": ending_before,
                        "from_ts": from_ts,
                        "limit": limit,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                        "starting_after": starting_after,
                        "to_ts": to_ts,
                    },
                    span_search_params.SpanSearchParams,
                ),
            ),
            model=Span,
            method="post",
        )

    async def upsert_batch(
        self,
        *,
        items: Iterable[span_upsert_batch_params.Item],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpanUpsertBatchResponse:
        """
        Upsert Spans in Batch

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/v5/spans/batch",
            body=await async_maybe_transform({"items": items}, span_upsert_batch_params.SpanUpsertBatchParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpanUpsertBatchResponse,
        )


class SpansResourceWithRawResponse:
    def __init__(self, spans: SpansResource) -> None:
        self._spans = spans

        self.create = to_raw_response_wrapper(
            spans.create,
        )
        self.retrieve = to_raw_response_wrapper(
            spans.retrieve,
        )
        self.update = to_raw_response_wrapper(
            spans.update,
        )
        self.batch = to_raw_response_wrapper(
            spans.batch,
        )
        self.search = to_raw_response_wrapper(
            spans.search,
        )
        self.upsert_batch = to_raw_response_wrapper(
            spans.upsert_batch,
        )


class AsyncSpansResourceWithRawResponse:
    def __init__(self, spans: AsyncSpansResource) -> None:
        self._spans = spans

        self.create = async_to_raw_response_wrapper(
            spans.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            spans.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            spans.update,
        )
        self.batch = async_to_raw_response_wrapper(
            spans.batch,
        )
        self.search = async_to_raw_response_wrapper(
            spans.search,
        )
        self.upsert_batch = async_to_raw_response_wrapper(
            spans.upsert_batch,
        )


class SpansResourceWithStreamingResponse:
    def __init__(self, spans: SpansResource) -> None:
        self._spans = spans

        self.create = to_streamed_response_wrapper(
            spans.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            spans.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            spans.update,
        )
        self.batch = to_streamed_response_wrapper(
            spans.batch,
        )
        self.search = to_streamed_response_wrapper(
            spans.search,
        )
        self.upsert_batch = to_streamed_response_wrapper(
            spans.upsert_batch,
        )


class AsyncSpansResourceWithStreamingResponse:
    def __init__(self, spans: AsyncSpansResource) -> None:
        self._spans = spans

        self.create = async_to_streamed_response_wrapper(
            spans.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            spans.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            spans.update,
        )
        self.batch = async_to_streamed_response_wrapper(
            spans.batch,
        )
        self.search = async_to_streamed_response_wrapper(
            spans.search,
        )
        self.upsert_batch = async_to_streamed_response_wrapper(
            spans.upsert_batch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["VectorStoreUpsertResponse"]


class VectorStoreUpsertResponse(BaseModel):
    """Response for batch insert/upsert operations."""

    failed: int
    """Failed operation count"""

    successful: int
    """Successful operation count"""

    total_attempted: int
    """Total operations attempted"""

    error_messages: Optional[List[str]] = None
    """Error messages from failed operations"""
